"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GreePlatform = void 0;
const dgram_1 = __importDefault(require("dgram"));
const crypto_1 = __importDefault(require("./crypto"));
const settings_1 = require("./settings");
const platformAccessory_1 = require("./platformAccessory");
function readLocaleMessages(locale) {
    try {
        return require(`./locale/${locale}`);
    }
    catch (err) {
        return require('./locale/en');
    }
}
/**
 * HomebridgePlatform
 * This class is the main constructor for your plugin, this is where you should
 * parse the user config and discover/register accessories with Homebridge.
 */
class GreePlatform {
    constructor(log, config, api) {
        this.log = log;
        this.api = api;
        this.Service = this.api.hap.Service;
        this.Characteristic = this.api.hap.Characteristic;
        this.handleMessage = (msg, rinfo) => {
            this.log.debug('handleMessage', msg.toString());
            try {
                const message = JSON.parse(msg.toString());
                if (message.i !== 1 || message.t !== 'pack') {
                    return;
                }
                const pack = crypto_1.default.decrypt(message.pack);
                if (message.t === 'pack' && pack.t === 'dev') {
                    this.registerDevice({
                        ...pack,
                        address: rinfo.address,
                        port: rinfo.port,
                    });
                }
            }
            catch (err) {
                this.log.info('message failed to handle', msg.toString());
                this.log.error('handleMessage Error', err);
            }
        };
        this.registerDevice = (deviceInfo) => {
            var _a;
            const deviceConfig = this.config.devices.find((item) => item.mac === deviceInfo.mac);
            let accessory = this.devices[deviceInfo.mac];
            if (deviceConfig === null || deviceConfig === void 0 ? void 0 : deviceConfig.disabled) {
                this.log.info(`accessory ${deviceInfo.mac} skipped`);
                if (accessory) {
                    this.api.unregisterPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
                    delete this.devices[deviceConfig.mac];
                }
                return;
            }
            if (accessory && this.initializedDevices[accessory.UUID]) {
                return;
            }
            if (!accessory) {
                const deviceName = (_a = deviceConfig === null || deviceConfig === void 0 ? void 0 : deviceConfig.name) !== null && _a !== void 0 ? _a : deviceInfo.name;
                this.log.debug(`Initializing new accessory ${deviceInfo.mac} with name ${deviceName}...`);
                const uuid = this.api.hap.uuid.generate(deviceInfo.mac);
                accessory = new this.api.platformAccessory(deviceInfo.mac, uuid);
                this.devices[deviceInfo.mac] = accessory;
                this.api.registerPlatformAccessories(settings_1.PLUGIN_NAME, settings_1.PLATFORM_NAME, [accessory]);
            }
            if (accessory) {
                // mark devices as initialized.
                accessory.context.device = deviceInfo;
                this.initializedDevices[accessory.UUID] = true;
                return new platformAccessory_1.GreeAirConditioner(this, accessory, deviceConfig);
            }
        };
        this.socket = dgram_1.default.createSocket('udp4');
        this.devices = {};
        this.initializedDevices = {};
        this.config = {
            ...settings_1.DEFAULT_PLATFORM_CONFIG,
            ...config,
            defaultValue: {
                ...settings_1.DEFAULT_DEVICE_CONFIG,
                ...(config.defaultValue || {}),
            },
        };
        this.log.debug('Config: %j', this.config);
        this.scanCount = 0;
        this.messages = readLocaleMessages(this.config.language).default;
        // When this event is fired it means Homebridge has restored all cached accessories from disk.
        // Dynamic Platform plugins should only register new accessories after this event was fired,
        // in order to ensure they weren't added to homebridge already. This event can also be used
        // to start discovery of new accessories.
        this.api.on('didFinishLaunching', () => {
            log.debug('Executed didFinishLaunching callback');
            this.socket.on('message', this.handleMessage);
            // run the method to discover / register your devices as accessories
            this.discoverDevices();
        });
        this.log.debug('Finished initializing platform:', this.config.name);
    }
    broadcastScan() {
        const message = Buffer.from(JSON.stringify({ t: 'scan' }));
        this.socket.send(message, 0, message.length, settings_1.UDP_SCAN_PORT, this.config.scanAddress, () => {
            this.log.debug(`Broadcast '${message}' ${this.config.scanAddress}:${settings_1.UDP_SCAN_PORT}`);
        });
    }
    configureAccessory(accessory) {
        var _a;
        this.log.info('Loading accessory from cache:', accessory.displayName, accessory.context.device);
        // add the restored accessory to the accessories cache so we can track if it has already been registered
        if ((_a = accessory.context.device) === null || _a === void 0 ? void 0 : _a.mac) {
            this.devices[accessory.context.device.mac] = accessory;
        }
    }
    discoverDevices() {
        this.socket.bind(this.config.port, () => {
            this.log.info(`UDP server bind to ${this.config.port}`);
            this.socket.setBroadcast(true);
            this.timer = setInterval(() => {
                this.broadcastScan();
                this.scanCount += 1;
                if (this.scanCount > this.config.scanCount && this.timer) {
                    this.log.info('Scan finished.');
                    clearInterval(this.timer);
                }
            }, this.config.scanTimeout);
        });
    }
}
exports.GreePlatform = GreePlatform;
//# sourceMappingURL=platform.js.map