"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GreeAirConditioner = void 0;
const crypto_1 = __importDefault(require("./crypto"));
const commands_1 = __importDefault(require("./commands"));
const settings_1 = require("./settings");
const HeaterCoolerToggleSwitch_1 = require("./HeaterCoolerToggleSwitch");
/**
 * Platform Accessory
 * An instance of this class is created for each accessory your platform registers
 * Each accessory may expose multiple services of different service types.
 */
class GreeAirConditioner {
    constructor(platform, accessory, deviceConfig) {
        this.platform = platform;
        this.accessory = accessory;
        this.deviceConfig = deviceConfig;
        this.handleMessage = (msg, rinfo) => {
            if (this.getAddress() === rinfo.address) {
                const message = JSON.parse(msg.toString());
                // this.platform.log.debug(`[${this.getDeviceLabel()}] handle message: %j`, message);
                const pack = crypto_1.default.decrypt(message.pack, message.i === 1 ? undefined : this.key);
                switch (pack.t) {
                    case 'bindok':
                        this.key = pack.key;
                        this.binded = true;
                        this.platform.log.info(`[${this.getDeviceLabel()}] Device binded. ${this.key}`);
                        this.afterBinded();
                        break;
                    case 'dat': // update status
                        this.updateStatus(fieldsToObject(pack.cols, pack.dat));
                        break;
                    case 'res': // command response
                        this.updateStatus(fieldsToObject(pack.opt, pack.p || pack.val));
                        break;
                }
            }
        };
        this.socket = platform.socket;
        this.binded = false;
        this.isPending = false;
        this.key = undefined;
        this.status = {};
        // register event handler
        this.socket.on('message', this.handleMessage);
        // try to bind device;
        this.sendBindRequest();
        // set accessory information
        this.Info = this.accessory
            .getService(this.platform.Service.AccessoryInformation)
            .setCharacteristic(this.platform.Characteristic.Manufacturer, this.accessory.context.device.brand)
            .setCharacteristic(this.platform.Characteristic.Model, this.accessory.context.device.name ||
            this.accessory.context.device.model ||
            'Gree')
            .setCharacteristic(this.platform.Characteristic.Name, this.getName())
            .setCharacteristic(this.platform.Characteristic.SerialNumber, this.accessory.context.device.mac)
            .setCharacteristic(this.platform.Characteristic.FirmwareRevision, this.accessory.context.device.ver);
        this.PowerSwitch = this.initSwitch('power', this.platform.messages.power);
        this.QuietModeSwitch = this.initSwitch('quietMode', this.platform.messages.quietMode);
        this.LightSwitch = this.initSwitch('light', this.platform.messages.light);
        this.PowerfulModeSwitch = this.initSwitch('powerfulMode', this.platform.messages.powerfulMode);
        this.VerticalSwing = this.initSwitch('verticalSwing', this.platform.messages.verticalSwing);
        this.HorizontalSwing = this.initSwitch('horizontalSwing', this.platform.messages.horizontalSwing);
        this.DryModeToggle = this.initSwitch('dryMode', this.platform.messages.dryMode);
        this.FanModeToggle = this.initSwitch('fanMode', this.platform.messages.fanMode);
        this.SleepModeToggle = this.initSwitch('sleepMode', this.platform.messages.sleepMode);
        this.xFanModeToggle = this.initSwitch('xFan', this.platform.messages.xFan);
        this.HeaterCooler =
            this.accessory.getService(this.platform.messages.mode) ||
                this.accessory.addService(this.platform.Service.HeaterCooler, this.platform.messages.mode);
        this.HeaterCooler.addOptionalCharacteristic(this.platform.Characteristic.ConfiguredName);
        this.HeaterCooler.setCharacteristic(this.platform.Characteristic.ConfiguredName, this.getConfiguredName(this.platform.messages.mode));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.Active)
            .on('get', this.getCharacteristic.bind(this, 'deviceActive'))
            .on('set', this.setCharacteristic.bind(this, 'deviceActive'));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.CurrentTemperature)
            .updateValue(25) // for initialize
            .on('get', this.getCharacteristic.bind(this, 'currentTemperature'));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.CurrentHeaterCoolerState).on('get', this.getCharacteristic.bind(this, 'currentState'));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.TargetHeaterCoolerState)
            .on('get', this.getCharacteristic.bind(this, 'targetState'))
            .on('set', this.setCharacteristic.bind(this, 'targetState'));
        // Optional Characteristic
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.CoolingThresholdTemperature)
            .setProps({
            minValue: this.getConfig('minimumTargetTemperature'),
            maxValue: this.getConfig('maximumTargetTemperature'),
            minStep: 0.1,
        })
            .updateValue(20)
            .on('get', this.getCharacteristic.bind(this, 'targetTemperature'))
            .on('set', this.setCharacteristic.bind(this, 'targetTemperature'));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.HeatingThresholdTemperature)
            .setProps({
            minValue: this.getConfig('minimumTargetTemperature'),
            maxValue: this.getConfig('maximumTargetTemperature'),
            minStep: 0.1,
        })
            .updateValue(25)
            .on('get', this.getCharacteristic.bind(this, 'targetTemperature'))
            .on('set', this.setCharacteristic.bind(this, 'targetTemperature'));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.TemperatureDisplayUnits)
            .on('get', this.getCharacteristic.bind(this, 'units'))
            .on('set', this.setCharacteristic.bind(this, 'units'));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.RotationSpeed)
            .setProps({
            minValue: 0,
            maxValue: 5,
            minStep: 1,
        })
            .on('get', this.getCharacteristic.bind(this, 'speed'))
            .on('set', this.setCharacteristic.bind(this, 'speed'));
        this.HeaterCooler.getCharacteristic(this.platform.Characteristic.SwingMode)
            .on('get', this.getCharacteristic.bind(this, 'verticalSwing'))
            .on('set', this.setCharacteristic.bind(this, 'verticalSwing'));
        const hasFanMode = this.getConfig('hasFanMode');
        if (hasFanMode) {
            this.Fan =
                this.accessory.getService(this.platform.messages.fanSpeed) ||
                    this.accessory.addService(this.platform.Service.Fanv2, this.platform.messages.fanSpeed);
            this.Fan.addOptionalCharacteristic(this.platform.Characteristic.ConfiguredName);
            this.Fan.setCharacteristic(this.platform.Characteristic.ConfiguredName, this.getConfiguredName(`${this.platform.messages.fanSpeed}(${this.platform.messages.fanMode})`));
            this.Fan.getCharacteristic(this.platform.Characteristic.Active)
                .on('get', this.getCharacteristic.bind(this, 'fanMode'))
                .on('set', this.setCharacteristic.bind(this, 'fanMode'));
            this.Fan.getCharacteristic(this.platform.Characteristic.RotationSpeed)
                .setProps({
                minValue: 0,
                maxValue: 5,
                minStep: 1,
            })
                .on('get', this.getCharacteristic.bind(this, 'speed'))
                .on('set', this.setCharacteristic.bind(this, 'speed'));
        }
        else {
            // clean up
            const fanService = this.accessory.getService(this.platform.messages.fanSpeed);
            if (fanService) {
                this.accessory.removeService(fanService);
            }
        }
    }
    getCharacteristic(key, callback) {
        const value = this[key];
        this.platform.log.debug(`[${this.getDeviceLabel()}] Get characteristic: ${key}, value: ${value}`);
        if (value === null || value !== value) {
            callback(new Error(`Failed to get characteristic value for key: ${key}`));
        }
        else {
            callback(null, value);
        }
    }
    setCharacteristic(key, value, callback) {
        this.platform.log.debug(`[${this.getDeviceLabel()}] Set characteristic: ${key} to value: ${value}`);
        this[key] = value;
        callback(null);
    }
    get power() {
        return this.status[commands_1.default.power.code] === commands_1.default.power.value.on;
    }
    // TRUE OR FALSE
    set power(value) {
        if (value === this.power) {
            return;
        }
        const power = value ? commands_1.default.power.value.on : commands_1.default.power.value.off;
        const command = { [commands_1.default.power.code]: power };
        if (this.status[commands_1.default.mode.code] !== undefined) {
            command[commands_1.default.mode.code] = this.status[commands_1.default.mode.code];
        }
        this.sendCommand(command);
    }
    get deviceActive() {
        if (!this.power) {
            return this.platform.Characteristic.Active.INACTIVE;
        }
        switch (this.status[commands_1.default.mode.code]) {
            case commands_1.default.mode.value.auto:
            case commands_1.default.mode.value.cool:
            case commands_1.default.mode.value.heat:
                return this.platform.Characteristic.Active.ACTIVE;
            default:
                return this.platform.Characteristic.Active.INACTIVE;
        }
    }
    set deviceActive(value) {
        if (this.deviceActive === value || this.isPending) {
            return;
        }
        this.isPending = true;
        this.sendCommand({
            [commands_1.default.power.code]: value === this.platform.Characteristic.Active.ACTIVE
                ? commands_1.default.power.value.on
                : commands_1.default.power.value.off,
        });
    }
    get currentTemperature() {
        if (this.status[commands_1.default.temperature.code] !== undefined) {
            const raw = this.status[commands_1.default.temperature.code];
            const temInCelsius = raw - (this.getConfig('sensorOffset') || 0);
            if (this.units ===
                this.platform.Characteristic.TemperatureDisplayUnits.FAHRENHEIT) {
                return (temInCelsius * 9) / 5 + 32;
            }
            return temInCelsius;
        }
        return this.status[commands_1.default.targetTemperature.code];
    }
    get currentState() {
        if (!this.power) {
            return this.platform.Characteristic.CurrentHeaterCoolerState.INACTIVE;
        }
        switch (this.status[commands_1.default.mode.code]) {
            case commands_1.default.mode.value.cool:
                return this.platform.Characteristic.CurrentHeaterCoolerState.COOLING;
            case commands_1.default.mode.value.heat:
                return this.platform.Characteristic.CurrentHeaterCoolerState.HEATING;
            default:
                return this.platform.Characteristic.CurrentHeaterCoolerState.IDLE;
        }
    }
    get targetState() {
        const mode = this.status[commands_1.default.mode.code];
        // only active in home app when Characteristic.Active is Active
        switch (mode) {
            case commands_1.default.mode.value.auto:
                return this.platform.Characteristic.TargetHeaterCoolerState.AUTO;
            case commands_1.default.mode.value.cool:
                return this.platform.Characteristic.TargetHeaterCoolerState.COOL;
            case commands_1.default.mode.value.heat:
                return this.platform.Characteristic.TargetHeaterCoolerState.HEAT;
            default:
                return this.platform.Characteristic.TargetHeaterCoolerState.AUTO;
        }
    }
    set targetState(value) {
        let mode;
        switch (value) {
            case this.platform.Characteristic.TargetHeaterCoolerState.AUTO:
                mode = commands_1.default.mode.value.auto;
                break;
            case this.platform.Characteristic.TargetHeaterCoolerState.HEAT:
                mode = commands_1.default.mode.value.heat;
                break;
            case this.platform.Characteristic.TargetHeaterCoolerState.COOL:
                mode = commands_1.default.mode.value.cool;
        }
        if (mode === undefined || mode === this.status[commands_1.default.mode.code]) {
            return;
        }
        const command = this.getModeCache(mode) || {
            [commands_1.default.mode.code]: mode,
        };
        // cache mode info;
        const currentMode = this.status[commands_1.default.mode.code];
        if (currentMode !== undefined) {
            this.cacheMode(currentMode);
        }
        this.sendCommand(command);
    }
    get targetTemperature() {
        const temperature = this.status[commands_1.default.targetTemperature.code];
        const offset = this.status[commands_1.default.temperatureOffset.code];
        if (temperature === undefined || offset === undefined) {
            return this.getConfig('maximumTargetTemperature');
        }
        const valueInF = (temperature + 0.5 * offset) * 2 + 27;
        if (this.units ===
            this.platform.Characteristic.TemperatureDisplayUnits.FAHRENHEIT) {
            return valueInF;
        }
        else {
            return Math.round(((valueInF - 32.0) * 5.0) / 9.0);
        }
    }
    set targetTemperature(value) {
        this.platform.log.debug(`set targetTemperature: ${value}`);
        if (value === this.targetTemperature) {
            return;
        }
        let SetTem;
        let TemRec;
        if (this.units ===
            this.platform.Characteristic.TemperatureDisplayUnits.FAHRENHEIT) {
            SetTem = Math.round(((value - 32.0) * 5.0) / 9.0);
            TemRec = Number(((value - 32.0) * 5.0) / 9.0 - SetTem > 0);
        }
        else {
            SetTem = Math.round(value);
            TemRec = Number(value - SetTem > 0);
        }
        this.sendCommand({
            [commands_1.default.units.code]: this.status[commands_1.default.units.code],
            [commands_1.default.targetTemperature.code]: SetTem,
            [commands_1.default.temperatureOffset.code]: TemRec,
        });
    }
    get units() {
        switch (this.status[commands_1.default.units.code]) {
            case commands_1.default.units.value.fahrenheit:
                return this.platform.Characteristic.TemperatureDisplayUnits.FAHRENHEIT;
            case commands_1.default.units.value.celsius:
                return this.platform.Characteristic.TemperatureDisplayUnits.CELSIUS;
        }
    }
    set units(value) {
        if (value === this.units) {
            return;
        }
        if (value !== undefined) {
            this.HeaterCooler.getCharacteristic(this.platform.Characteristic.TemperatureDisplayUnits).updateValue(value);
        }
        const command = (() => {
            switch (value) {
                case this.platform.Characteristic.TemperatureDisplayUnits.CELSIUS:
                    return commands_1.default.units.value.celsius;
                case this.platform.Characteristic.TemperatureDisplayUnits.FAHRENHEIT:
                    return commands_1.default.units.value.fahrenheit;
            }
        })();
        this.sendCommand({ [commands_1.default.units.code]: command });
    }
    get speed() {
        const speed = this.status[commands_1.default.speed.code];
        return speed || 0;
        // const speed = this.status[commands.speed.code] as number;
        // const quietMode = this.status[commands.quietMode.code];
        // const powerfulMode = this.status[commands.powerfulMode.code];
        // if (
        //   quietMode === commands.quietMode.value.on &&
        //   speed === commands.speed.value.low
        // ) {
        //   return 1;
        // } else if (
        //   powerfulMode === commands.quietMode.value.on &&
        //   speed === commands.speed.value.high
        // ) {
        //   return 7;
        // } else if (speed === 0) {
        //   return speed;
        // } else {
        //   return speed + 1;
        // }
    }
    set speed(value) {
        if (value === this.speed) {
            return;
        }
        if (!this.power) {
            return;
        }
        this.sendCommand({
            [commands_1.default.quietMode.code]: commands_1.default.quietMode.value.off,
            [commands_1.default.powerfulMode.code]: commands_1.default.powerfulMode.value.off,
            [commands_1.default.speed.code]: value,
        });
        // if (value === 0) {
        //   // 自动
        //   this.sendCommand({
        //     [commands.quietMode.code]: commands.quietMode.value.off,
        //     [commands.powerfulMode.code]: commands.powerfulMode.value.off,
        //     [commands.speed.code]: commands.speed.value.auto,
        //   });
        // } else if (value === 1) {
        //   // 静音
        //   this.sendCommand({
        //     [commands.quietMode.code]: commands.quietMode.value.on,
        //     [commands.powerfulMode.code]: commands.powerfulMode.value.off,
        //     [commands.speed.code]: commands.speed.value.low,
        //   });
        // } else if (value === 7) {
        //   // 强劲
        //   this.sendCommand({
        //     [commands.quietMode.code]: commands.quietMode.value.off,
        //     [commands.powerfulMode.code]: commands.powerfulMode.value.on,
        //     [commands.speed.code]: commands.speed.value.high,
        //   });
        // } else {
        //   this.sendCommand({
        //     [commands.quietMode.code]: commands.quietMode.value.off,
        //     [commands.powerfulMode.code]: commands.powerfulMode.value.off,
        //     [commands.speed.code]: value - 1,
        //   });
        // }
    }
    get light() {
        return this.status[commands_1.default.light.code] === commands_1.default.light.value.on;
    }
    set light(value) {
        if (value === this.light) {
            return;
        }
        const command = {
            [commands_1.default.light.code]: value
                ? commands_1.default.light.value.on
                : commands_1.default.light.value.off,
        };
        this.sendCommand(command);
    }
    get quietMode() {
        return (this.status[commands_1.default.quietMode.code] === commands_1.default.quietMode.value.on &&
            this.status[commands_1.default.speed.code] === commands_1.default.speed.value.low);
    }
    set quietMode(value) {
        if (value === this.quietMode) {
            return;
        }
        if (value) {
            this.sendCommand({
                [commands_1.default.quietMode.code]: commands_1.default.quietMode.value.on,
                [commands_1.default.speed.code]: commands_1.default.speed.value.low,
                [commands_1.default.powerfulMode.code]: commands_1.default.powerfulMode.value.off,
            });
        }
        else {
            this.sendCommand({
                [commands_1.default.quietMode.code]: commands_1.default.quietMode.value.off,
                [commands_1.default.speed.code]: this.getConfig('defaultSpeed'),
                [commands_1.default.powerfulMode.code]: commands_1.default.powerfulMode.value.off,
            });
        }
    }
    get powerfulMode() {
        return (this.status[commands_1.default.powerfulMode.code] ===
            commands_1.default.powerfulMode.value.on &&
            this.status[commands_1.default.speed.code] === commands_1.default.speed.value.high);
    }
    set powerfulMode(value) {
        if (value === this.powerfulMode) {
            return;
        }
        if (value) {
            this.sendCommand({
                [commands_1.default.powerfulMode.code]: commands_1.default.powerfulMode.value.on,
                [commands_1.default.speed.code]: commands_1.default.speed.value.high,
                [commands_1.default.quietMode.code]: commands_1.default.quietMode.value.off,
            });
        }
        else {
            this.sendCommand({
                [commands_1.default.powerfulMode.code]: commands_1.default.powerfulMode.value.off,
                [commands_1.default.speed.code]: this.getConfig('defaultSpeed'),
                [commands_1.default.quietMode.code]: commands_1.default.quietMode.value.off,
            });
        }
    }
    get sleepMode() {
        return this.status[commands_1.default.sleepMode.code] === commands_1.default.sleepMode.value.on;
    }
    set sleepMode(value) {
        if (value === this.sleepMode) {
            return;
        }
        if (value) {
            this.sendCommand({
                [commands_1.default.sleepMode.code]: commands_1.default.sleepMode.value.on,
            });
        }
        else {
            this.sendCommand({
                [commands_1.default.sleepMode.code]: commands_1.default.sleepMode.value.off,
            });
        }
    }
    get xFan() {
        return this.status[commands_1.default.xFan.code] === commands_1.default.xFan.value.on;
    }
    set xFan(value) {
        // X-FAN在自动，风扇或加热模式下不可用
        switch (this.status[commands_1.default.mode.code]) {
            case commands_1.default.mode.value.auto:
            case commands_1.default.mode.value.fan:
            case commands_1.default.mode.value.heat:
                break;
            default:
                if (value) {
                    this.sendCommand({
                        [commands_1.default.xFan.code]: commands_1.default.xFan.value.on,
                    });
                }
                else {
                    this.sendCommand({
                        [commands_1.default.xFan.code]: commands_1.default.xFan.value.off,
                    });
                }
        }
    }
    get verticalSwing() {
        return (this.status[commands_1.default.swingVertical.code] ===
            commands_1.default.swingVertical.value.full);
    }
    set verticalSwing(value) {
        if (value === this.verticalSwing) {
            return;
        }
        if (value) {
            this.sendCommand({
                [commands_1.default.swingVertical.code]: commands_1.default.swingVertical.value.full,
            });
        }
        else {
            this.sendCommand({
                [commands_1.default.swingVertical.code]: this.getConfig('defaultVerticalSwing'),
            });
        }
    }
    get horizontalSwing() {
        return (this.status[commands_1.default.swingHorizontal.code] ===
            commands_1.default.swingHorizontal.value.full);
    }
    set horizontalSwing(value) {
        if (value === this.horizontalSwing) {
            return;
        }
        if (value) {
            this.sendCommand({
                [commands_1.default.swingHorizontal.code]: commands_1.default.swingHorizontal.value.full,
            });
        }
        else {
            this.sendCommand({
                [commands_1.default.swingHorizontal.code]: this.getConfig('defaultHorizontalSwing'),
            });
        }
    }
    get dryMode() {
        return (this.status[commands_1.default.mode.code] === commands_1.default.mode.value.dry &&
            this.status[commands_1.default.power.code] === commands_1.default.power.value.on);
    }
    set dryMode(value) {
        if (!!value === this.dryMode) {
            return;
        }
        if (value) {
            // cache current mode with key: last
            if (this.power &&
                this.status[commands_1.default.mode.code] !== commands_1.default.mode.value.dry) {
                this.cacheMode('last');
            }
            this.sendCommand({
                [commands_1.default.mode.code]: commands_1.default.mode.value.dry,
                [commands_1.default.power.code]: commands_1.default.power.value.on,
            });
        }
        else {
            const lastState = this.getModeCache('last', true);
            if (lastState) {
                this.sendCommand(lastState);
            }
            else {
                // power off
                this.sendCommand({
                    [commands_1.default.power.code]: commands_1.default.power.value.off,
                });
            }
        }
    }
    get fanMode() {
        return (this.status[commands_1.default.mode.code] === commands_1.default.mode.value.fan &&
            this.status[commands_1.default.power.code] === commands_1.default.power.value.on);
    }
    set fanMode(value) {
        if (!!value === this.fanMode) {
            return;
        }
        if (value) {
            // cache current mode with key: last
            if (this.power &&
                this.status[commands_1.default.mode.code] !== commands_1.default.mode.value.fan) {
                this.cacheMode('last');
            }
            this.sendCommand({
                [commands_1.default.mode.code]: commands_1.default.mode.value.fan,
                [commands_1.default.power.code]: commands_1.default.power.value.on,
            });
        }
        else {
            const lastState = this.getModeCache('last', true);
            if (lastState) {
                this.sendCommand(lastState);
            }
            else {
                // power off
                this.sendCommand({
                    [commands_1.default.power.code]: commands_1.default.power.value.off,
                });
            }
        }
    }
    sendMessage(message) {
        const pack = crypto_1.default.encrypt(message, this.key);
        const payload = {
            cid: 'app',
            i: this.key === undefined ? 1 : 0,
            t: 'pack',
            uid: 0,
            tcid: this.getMac(),
            pack,
        };
        this.platform.log.debug(`[${this.getDeviceLabel()}] send request: %j, %j`, payload, message);
        try {
            this.platform.socket.send(JSON.stringify(payload), this.getPort(), this.getAddress());
        }
        catch (err) {
            this.platform.log.error(err.message);
        }
    }
    sendBindRequest() {
        const message = {
            mac: this.getMac(),
            t: 'bind',
            uid: 0,
        };
        this.platform.log.debug(`Bind to device: ${this.getMac()}`);
        this.sendMessage(message);
    }
    requestDeviceStatus() {
        const message = {
            mac: this.getMac(),
            t: 'status',
            cols: this.getCols(),
        };
        this.sendMessage(message);
    }
    sendCommand(commands) {
        this.platform.log.debug(`[${this.getDeviceLabel()}] Send commands: %j`, commands);
        const keys = Object.keys(commands);
        const values = keys.map((k) => commands[k]);
        const message = {
            t: 'cmd',
            opt: keys,
            p: values,
        };
        this.sendMessage(message);
    }
    updateStatus(patch) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
        if (this.platform.config.logStatus) {
            this.platform.log.info(`[${this.getDeviceLabel()}] Update Status: %j`, patch);
        }
        else {
            this.platform.log.debug(`[${this.getDeviceLabel()}] Update Status: %j`, patch);
        }
        this.status = {
            ...this.status,
            ...patch,
        };
        this.isPending = false;
        if (patch[commands_1.default.power.code] !== undefined) {
            (_a = this.PowerSwitch) === null || _a === void 0 ? void 0 : _a.update();
            this.HeaterCooler.getCharacteristic(this.platform.Characteristic.Active).updateValue(this.deviceActive);
        }
        if (patch[commands_1.default.units.code] !== undefined) {
            this.HeaterCooler.getCharacteristic(this.platform.Characteristic.CurrentTemperature).updateValue(this.currentTemperature);
            this.HeaterCooler.getCharacteristic(this.platform.Characteristic.TargetTemperature).updateValue(this.targetTemperature);
        }
        // update accessory characteristic
        if (patch[commands_1.default.quietMode.code] !== undefined ||
            patch[commands_1.default.powerfulMode.code] !== undefined) {
            (_b = this.QuietModeSwitch) === null || _b === void 0 ? void 0 : _b.update();
            (_c = this.PowerfulModeSwitch) === null || _c === void 0 ? void 0 : _c.update();
            this.HeaterCooler.getCharacteristic(this.platform.Characteristic.RotationSpeed).updateValue(this.speed);
            (_d = this.Fan) === null || _d === void 0 ? void 0 : _d.getCharacteristic(this.platform.Characteristic.RotationSpeed).updateValue(this.speed);
        }
        if (patch[commands_1.default.mode.code] !== undefined) {
            this.HeaterCooler.getCharacteristic(this.platform.Characteristic.TargetHeaterCoolerState).updateValue(this.targetState);
            (_e = this.DryModeToggle) === null || _e === void 0 ? void 0 : _e.update();
            (_f = this.FanModeToggle) === null || _f === void 0 ? void 0 : _f.update();
        }
        if (patch[commands_1.default.swingHorizontal.code] !== undefined) {
            (_g = this.HorizontalSwing) === null || _g === void 0 ? void 0 : _g.update();
        }
        if (patch[commands_1.default.swingVertical.code] !== undefined) {
            (_h = this.VerticalSwing) === null || _h === void 0 ? void 0 : _h.update();
        }
        if (patch[commands_1.default.sleepMode.code] !== undefined) {
            (_j = this.SleepModeToggle) === null || _j === void 0 ? void 0 : _j.update();
        }
        if (patch[commands_1.default.xFan.code] !== undefined) {
            (_k = this.xFanModeToggle) === null || _k === void 0 ? void 0 : _k.update();
        }
        if (patch[commands_1.default.light.code] !== undefined) {
            (_l = this.LightSwitch) === null || _l === void 0 ? void 0 : _l.update();
        }
    }
    afterBinded() {
        if (this.updateTimer) {
            clearInterval(this.updateTimer);
        }
        this.requestDeviceStatus();
        this.updateTimer = setInterval(() => {
            this.requestDeviceStatus();
        }, this.getConfig('statusUpdateInterval') * 1000);
    }
    getMac() {
        return this.accessory.context.device.mac;
    }
    getAddress() {
        return this.accessory.context.device.address;
    }
    getPort() {
        return this.accessory.context.device.port;
    }
    getName() {
        var _a;
        return ((_a = this.deviceConfig) === null || _a === void 0 ? void 0 : _a.name) || this.accessory.context.device.name;
    }
    getCols() {
        // TODO: may config features based on some static database;
        if (!this.cols) {
            this.cols = Object.keys(commands_1.default).map((k) => commands_1.default[k].code);
        }
        return this.cols;
    }
    getDeviceLabel() {
        return `${this.getMac()} -- ${this.getAddress()}:${this.getPort()}`;
    }
    getConfig(key) {
        var _a, _b, _c, _d;
        return ((_d = (_b = (_a = this.deviceConfig) === null || _a === void 0 ? void 0 : _a[key]) !== null && _b !== void 0 ? _b : (_c = this.platform.config.defaultValue) === null || _c === void 0 ? void 0 : _c[key]) !== null && _d !== void 0 ? _d : settings_1.DEFAULT_PLATFORM_CONFIG.defaultValue[key]);
    }
    cacheMode(key) {
        if (!this.accessory.context.modeStates) {
            this.accessory.context.modeStates = {};
        }
        this.accessory.context.modeStates[key] = {
            [commands_1.default.mode.code]: this.status[commands_1.default.mode.code],
            [commands_1.default.targetTemperature.code]: this.status[commands_1.default.targetTemperature.code],
            [commands_1.default.speed.code]: this.status[commands_1.default.speed.code],
            [commands_1.default.quietMode.code]: this.status[commands_1.default.quietMode.code],
            [commands_1.default.powerfulMode.code]: this.status[commands_1.default.powerfulMode.code],
        };
        this.platform.log.debug(`[${this.getDeviceLabel()}] cache status: %s, %j`, key, this.accessory.context.modeStates[key]);
    }
    getModeCache(key, clear = false) {
        var _a;
        const cache = (_a = this.accessory.context.modeStates) === null || _a === void 0 ? void 0 : _a[key];
        this.platform.log.debug(`[${this.getDeviceLabel()}] get mode cache: %s, %j`, key, cache);
        if (cache && clear) {
            delete this.accessory.context.modeStates[key];
        }
        return cache;
    }
    initSwitch(name, displayName) {
        this.platform.log.debug(`[${this.getDeviceLabel()}] try initSwitch: %s`, name);
        const switches = this.getConfig('switches');
        const hasFanMode = this.getConfig('hasFanMode');
        const hasDryMode = this.getConfig('hasDryMode');
        this.platform.log.debug(`[${this.getDeviceLabel()}] switches: %s`, switches);
        const shouldInit = switches.split(',').indexOf(name) > -1 &&
            (hasFanMode || name !== 'fanMode') &&
            (hasDryMode || name !== 'dryMode');
        if (shouldInit) {
            this.platform.log.debug(`[${this.getDeviceLabel()}] initSwitch: %s`, name);
            return new HeaterCoolerToggleSwitch_1.HeaterCoolerToggleSwitch(this, name, displayName);
        }
        else {
            const service = this.accessory.getService(displayName);
            if (service) {
                this.platform.log.debug(`[${this.getDeviceLabel()}] cleanup: %s`, name);
                this.accessory.removeService(service);
            }
        }
    }
    getConfiguredName(name) {
        const subAccessoryPrefix = this.getConfig('subAccessoryPrefix');
        if (subAccessoryPrefix === '<device>') {
            return `${this.getName()} ${name}`;
        }
        const match = /<device>\[(-?\d?):(-?\d?)\]/.exec(subAccessoryPrefix);
        if (match) {
            const start = match[1] ? Number(match[1]) : 0;
            const end = match[2] ? Number(match[2]) : undefined;
            const prefix = this.getName().substring(start, end);
            return `${prefix} ${name}`;
        }
        if (subAccessoryPrefix) {
            return `${subAccessoryPrefix} ${name}`;
        }
        return name;
    }
}
exports.GreeAirConditioner = GreeAirConditioner;
const fieldsToObject = (cols, values) => {
    const obj = {};
    cols.forEach((key, i) => {
        obj[key] = values[i];
    });
    return obj;
};
//# sourceMappingURL=platformAccessory.js.map